import {Component} from "react";

export declare module QuantumMetricLibrary  {
    interface InitializationOptions {
        testModeEnabled: boolean;
        sslPinningEnabled: boolean;
        certificateKey?: string;
        crashReportingEnabled: boolean;
    }

    function init(): void
    function initialize(subscription: string, uid: string): void;
    function initializeWithOptions(subscription: string, uid: string, options: InitializationOptions)
    function sendNewPageNamed(name: string): void;
    function sendEvent (eventId: number, value?: string, flags?: number): void;
    function sendMultidimensionalEvent (eventId: number, value?: string, json?: string, flags?: number): void;
    function pauseQM(): void
    function resumeQM(): void
    function stopQM(): void
    function resetSessionQM(): void
    function setSessionCookieCallbackQM(callback: (sessionCookie: string, userString: string, replayUrl?: string) => any): void
    function enableTestConfig(enabled: boolean): void;
}

type PrivacyTypes = 'mask'|'encrypt'|'unmask'|'no-encrypt'|'capture';
type QuantumComponentProps = {
    children?: React.ReactNode,
    privacy: PrivacyTypes,
    style?: React.CSSProperties;
};

export const QuantumMetricLibrary: any.eventType = Object.freeze({
    EventTypeConversion:(1 << 0), // => 1
    EventTypeLogin:(1 << 1), // => 2
    EventTypeEmail:(1 << 2),  // => 4
    EventTypeFirstName:(1 << 3),  // => 8
    EventTypeLastName:(1 << 4),  // => 16
    EventTypeCartValue:(1 << 6),  // => 64
    EventTypeABN:(1 << 7),
    EventTypeEncrypted:(1 << 8) // => 256
    //internal events below
    // EventTypeQMVersion:(1 << 9),
    // EventTypeApplicationType:(1 << 10),
    // EventTypeApplicationVersion :(1 << 11)
})

export const QuantumComponent: React.FC<QuantumComponentProps>;